//************************************************************************
//*                           BipNUM Win V0.9.2                          *
//*----------------------------------------------------------------------*
//*                   BipNUM Class for Microsoft Windows                 *
//*                                                                      *
//*                                                                      *
//* Licence : NFM                                                        *
//*                                                         P. Basset    *
//************************************************************************


#include <string>
#include <sstream>
#include "BipNUM.h"


// error codes for BP_Currency::iBPError ( return by BP_Currency::geterror() )
//
//  0 = no error
//  1 = malformed currency string in set command
//  2 = unknown standard format in setstdformat command
//  3 = format too small to display integer part
//  4 = integer format is bad
//  5 = decimal format is bad
//  6 = number is too big
//  7 = integer part overload
//  8 = decimal part overload - internal error



//-----------------------------------------------------------------
//!    BP_Currency::BP_Currency : class constructor               !
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Currency::BP_Currency ()
{

	// reset currency and error code
	BP_Currency::reset ();

}



//-----------------------------------------------------------------
//!    BP_Currency::~BP_Currency : class destructor               !
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Currency::~BP_Currency ()
{
}



//-----------------------------------------------------------------
//!    BP_Currency::reset : reset currency and format             !
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Currency::reset (void)
{
	// reset currency value
	BP_Currency::setnull ();

	// default format is "US"
	BP_Currency::setstdformat (0, 0, "US");

	// reset error code
	iBPError = 0;
}



//-----------------------------------------------------------------
//!    BP_Currency::copyvalue : copy value of BP_Currency         !
//-----------------------------------------------------------------
//! BP_CurSource : source BP_Currency                             !
//!																  !
//! return : none                                                 !
//-----------------------------------------------------------------
//! rem : copy only the value of the currency,                    !
//!       don't copy the format                                   !
//-----------------------------------------------------------------
BP_Currency::copyvalue (BP_Currency BP_CurSource)
{
	// reset error code
	iBPError = 0;

	// copy value
	BP_Cur = BP_CurSource.BP_Cur;
}



//-----------------------------------------------------------------
//!    BP_Currency::copyformat : copy format of BP_Currency       !
//-----------------------------------------------------------------
//! BP_CurSource : source BP_Currency                             !
//!																  !
//! return : none                                                 !
//-----------------------------------------------------------------
//! rem : copy only the format of the currency,                   !
//!       don't copy the value                                    !
//-----------------------------------------------------------------
BP_Currency::copyformat (BP_Currency BP_CurSource)
{
	// reset error code
	iBPError = 0;

	// copy format
	BP_Fmt = BP_CurSource.BP_Fmt;
}



//-----------------------------------------------------------------
//!    BP_Currency::set : fill string currency into a BP_Currency !
//!                           container                           !
//-----------------------------------------------------------------
//! BP_SetString : currency value                                 !
//!																  !
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Currency::set (const std::string sBP_SetString)
{

	int iTemp;
	std::string sInput;
	std::string sString;

	
	// reset error code
	iBPError = 0;

	// transfert input string
	sInput = sBP_SetString;

	// if input string is not null
	if (sInput != "" && sInput != BP_Fmt.sBP_Null)
	{
		// delete currency symbol
		if (BP_Fmt.sBP_CurrencySymbol != "")
		{
			iTemp = sInput.find (BP_Fmt.sBP_CurrencySymbol);
			if (iTemp >= 0)
					sInput.erase(iTemp, BP_Fmt.sBP_CurrencySymbol.length());
		}

		// delete all leading char
		if (BP_Fmt.sBP_LeadChar != "")
		{
			iTemp = sInput.find (BP_Fmt.sBP_LeadChar);
			while (iTemp >= 0)
			{
				sInput.erase(iTemp, 1);
				iTemp = sInput.find (BP_Fmt.sBP_LeadChar);
			}
		}
		
		// delete all kilo sign
		if (BP_Fmt.sBP_KiloSign != "")
		{
			iTemp = sInput.find (BP_Fmt.sBP_KiloSign);
			while (iTemp >= 0)
			{
				sInput.erase(iTemp, BP_Fmt.sBP_KiloSign.length());
				iTemp = sInput.find (BP_Fmt.sBP_KiloSign);
			}
		}

		// delete leading spaces
		if (sInput [0] == ' ')
		{
			iTemp = sInput.find_first_not_of (" ");
			sInput = sInput.substr (iTemp);
		}
		
		// delete ending spaces
		iTemp = sInput.length() - 1;
		if (sInput [iTemp] == ' ')
		{
			while (iTemp >= 0 && sInput [iTemp] == ' ')
			{
				sInput.erase(iTemp, 1);
				iTemp--;
			}
		}

		// look for negative sign : "-"
		BP_Cur.bBP_Negative = false;
		iTemp = sInput.find ("-");
		if (iTemp >= 0)
		{
			BP_Cur.bBP_Negative = true;

			// erase negative sign
			sInput.erase(iTemp, 1);
		}

		// delete positive sign : "+"
		iTemp = sInput.find ("+");
		if (iTemp >= 0)
			sInput.erase(iTemp, 1);



		// verify if input string is numeric (the format was deleted before)
		sString = sInput;
		if (BP_Fmt.sBP_DecimalSign != "")
		{
			iTemp = sString.find (BP_Fmt.sBP_DecimalSign);
			if (iTemp >= 0)
				// delete temporary decimal sign
				sString.erase (iTemp, BP_Fmt.sBP_DecimalSign.length());
		}
		else
		{
			// suppose decimal sign is "."
			iTemp = sString.find (".");
			if (iTemp >= 0)
				// delete temporary decimal sign
				sString.erase(iTemp, 1);
			else
			// not found, so try with ","
			{
				iTemp = sString.find (",");
				if (iTemp >= 0)
					// delete temporary decimal sign
					sString.erase(iTemp, 1);
			}
		}

		iTemp = 0;
		while (iTemp < sString.length() && iBPError == 0)
		{
			if (sString [iTemp] < '0' || sString [iTemp] > '9')
				// input string is not numeric
				iBPError = 1;

			iTemp++;
		}



		// store currency value
		if (iBPError == 0)
		{
			// look for decimal sign
			iTemp = sInput.find (BP_Fmt.sBP_DecimalSign);

			// no decimal part
			if (iTemp < 1)
			{
				if (sInput.length () <= 18)
				{
					BP_Cur.lBP_Integer = _atoi64 (sInput.c_str());
					BP_Cur.lBP_Decimal = 0;
					BP_Cur.bBP_Null = false;
				}
				else
				{
					// error : set currency to null
					BP_Currency::setnull ();
					iBPError = 6;
				}

			}
			// number as a decimal part
			else
			{
				// integer part has less than 18 digits
				if (iTemp <= 18)
				{
					// store integer part
					sString = sInput.substr (0, iTemp);
					BP_Cur.lBP_Integer = _atoi64 (sString.c_str());

					// decimal part has less than 9 digits (+1)
					if ((sInput.length () - iTemp) <= 10)
					{
						// store decimal part
						sString = "1" + sInput.substr (iTemp + BP_Fmt.sBP_DecimalSign.length());
						sString += "0000000000";
						sString = sString.substr (0, BP_DEC_DIGIT + 1);
						BP_Cur.lBP_Decimal = atoi (sString.c_str());

						// decimal part is null
						if (BP_Cur.lBP_Decimal == BP_DECIMAL)
							BP_Cur.lBP_Decimal = 0;
					}
					else
					{
						// error : set currency to null
						BP_Currency::setnull ();
						iBPError = 6;
					}

					// currency is not null
					BP_Cur.bBP_Null = false;
				}
				// integer part has more than 18 digits
				else
				{
					// error : set currency to null
					BP_Currency::setnull ();
					iBPError = 6;
				}
			}
		}
		else
		{
			// error : set currency to null
			BP_Currency::setnull ();
		}
	}
	else
	{
		// currency is null
		BP_Currency::setnull ();
	}

}



//-----------------------------------------------------------------
//!    BP_Currency::setnull : set currency to null value          !
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Currency::setnull (void)
{
	// reset error code - no, keep error code
//	iBPError = 0;

	// set currency to null
	BP_Cur.lBP_Integer = 0;
	BP_Cur.lBP_Decimal = 0;
	BP_Cur.bBP_Null = true;
	BP_Cur.bBP_Negative = false;
}



//-----------------------------------------------------------------
//!    BP_Currency::geterror : return error code                  !
//-----------------------------------------------------------------
//! return : error code or 0 if none                              !
//-----------------------------------------------------------------
int BP_Currency::geterror (void)
{

	return BP_Currency::iBPError;

}



//-----------------------------------------------------------------
//!    BP_Currency::str : return currency into a string           !
//!                        depending on format                    !
//-----------------------------------------------------------------
//! return : string                                               !
//-----------------------------------------------------------------
//! use BP_Currency::setformat before BP_Currency::str            !
//! if error occurs, the return string = "" and                   !
//!     BP_Currency:geterror() return error code                  !
//-----------------------------------------------------------------
std::string BP_Currency::str (void)
{
	std::ostringstream os;
	char c255 [255];
	std::string sString;
	int iTemp;


	// reset error code
	iBPError = 0;

	if (BP_Cur.bBP_Null != true)
	{
		// reset working string
		sString = "";

		// add 50 leading zero
		if (BP_Fmt.sBP_LeadChar != "")
			sString.append (50, BP_Fmt.sBP_LeadChar [0]);

		// display negative sign : "-"
		iTemp = 0;
		if (BP_Cur.bBP_Negative == true)
		{
			sString += "-";
			iTemp = 1;
		}
		
		// convert long long to string
		_i64toa (BP_Cur.lBP_Integer, c255, 10);
		sString += c255;

		// limit to format length
		iTemp += BP_Fmt.iBP_NumInt;
		if (iTemp < sString.length())
		{
			sString = sString.substr (sString.length() - iTemp, iTemp);
			iBPError = 3;
		}
		
		// insert kilo sign
		if (BP_Fmt.sBP_KiloSign != "")
		{
			iTemp = sString.length() - 3;
			while (iTemp > 0)
			{
				//
				if (sString [iTemp] != ' ' && sString [iTemp] != BP_Fmt.sBP_LeadChar [0])
					sString.insert(iTemp, BP_Fmt.sBP_KiloSign);

				iTemp -= 3;
			}
		}

		// display integer part
		os << sString;


		// if decimal part exists
		if (BP_Fmt.iBP_NumDec != 0 && BP_Cur.lBP_Decimal != 0 &&
			BP_Cur.lBP_Decimal != BP_DECIMAL)
		{

			// display decimal sign
			if (BP_Fmt.iBP_NumDec > 0)
				os << BP_Fmt.sBP_DecimalSign;

			//convert decimal part to string
			itoa (BP_Cur.lBP_Decimal, c255, 10);
			sString = c255;

			if (BP_Fmt.bBP_TrailZero)
			{
				// add trailing zero and truncate decimal part
				sString += "000000000000";
				sString = sString.substr (0, BP_Fmt.iBP_NumDec + 1);
			}
			else
			{
				// delete trailing zero
				iTemp = sString.find_last_not_of ("0", sString.length());
				if (iTemp != -1)
					sString = sString.substr (0, iTemp + 1);
			}
			
			// display decimal part, exclude first char
			os << sString.substr (1);
		}

		// display currency symbol
		if (BP_Fmt.sBP_CurrencySymbol != "")
			os << " " << BP_Fmt.sBP_CurrencySymbol;

		return os.str();

	}
	else
		// currency is null;
		return BP_Fmt.sBP_Null;

}



//-----------------------------------------------------------------
//!    BP_Currency::isnull : return true if currency is null      !
//!                           else return false                   !
//-----------------------------------------------------------------
//! return : bool                                                 !
//-----------------------------------------------------------------
bool BP_Currency::isnull (void)
{
	// reset error code - no, keep error code
//	iBPError = 0;

	return BP_Cur.bBP_Null;

}



//-----------------------------------------------------------------
//!    BP_Currency::setformat : set currency format               !
//!                         use by BP_Currency::str               !
//-----------------------------------------------------------------
//! iBP_SetNumInt : character to display before decimal sign      !
//! iBP_SetNumDec : character to display after decimal sign       !
//! sBP_SetDecimalSign : decimal sign (default ".")               !
//! sBP_SetKiloSign : kilo sign (ie : " " , "." or "")            !
//! sBP_SetCurrencySymbol : currency symbol (ie : "euro" or "$")  !
//! sBP_SetLeadChar : leading char display before number          !
//!                      (truncated to 1 char)                    !
//! sBP_SetNull : null format                                     !
//! bBP_TrailZero : display trailing zeros                        !
//!                                                               !
//! samples :                                                     !
//! setformat(4,2,",",".","$","","") give "1.234,56 $"            !
//! setformat(3,0,"","","","0","") give "001"                     !
//! setformat(1,0,"","","","","NULL") give "NULL"                 !
//! setformat(10,2,",","","$","*","") give "******1234,56 $"      !
//!                                                               !
//! return : none                                                 !
//-----------------------------------------------------------------
//!     BP_Date::geterror() return error code                     !
//-----------------------------------------------------------------
BP_Currency::setformat (const int iBP_SetNumInt,
						const int iBP_SetNumDec,
						const std::string sBP_SetDecimalSign,
						const std::string sBP_SetKiloSign,
						const std::string sBP_SetCurrencySymbol,
						const std::string sBP_SetLeadChar,
						const std::string sBP_SetNull,
						const bool bBP_TrailZero)
{
	// reset error code
	iBPError = 0;

	if (iBP_SetNumInt < 0 || iBP_SetNumInt > 18)
		iBPError = 4;

	if (iBP_SetNumDec < 0 || iBP_SetNumDec > 9)
		iBPError = 5;

	// store values
	BP_Fmt.iBP_NumInt = iBP_SetNumInt;
	BP_Fmt.iBP_NumDec = iBP_SetNumDec;
	BP_Fmt.sBP_DecimalSign = sBP_SetDecimalSign;
	BP_Fmt.sBP_KiloSign = sBP_SetKiloSign;
	BP_Fmt.sBP_CurrencySymbol = sBP_SetCurrencySymbol;
	BP_Fmt.sBP_LeadChar = sBP_SetLeadChar.substr (0, 1);
	BP_Fmt.sBP_Null = sBP_SetNull;
	BP_Fmt.bBP_TrailZero = bBP_TrailZero;
}



//-----------------------------------------------------------------
//!    BP_Currency::setformat : set currency format               !
//!                         for simple format                     !
//-----------------------------------------------------------------
//! iBP_SetFormat : format of the currency                        !
//!                                                               !
//! format :                                                      !
//!   char          indicate                                      !
//!    9       one numeric char                                   !
//!    8       one decimal char                                   ! 
//!    x       decimal sign (x = any char except 9 and 8          !
//! order : integer char, decimal sign, decimal char              !
//!                                                               !
//! samples :                                                     !
//! setformat("9999.88") give "1234.56"                           !
//! setformat("9,888") give "1,234"                               !
//! setformat("99") give "12"                                     !
//!                                                               !
//! return : none                                                 !
//-----------------------------------------------------------------
//!     BP_Date::geterror() return error code                     !
//-----------------------------------------------------------------
BP_Currency::setformat (const std::string sBP_SetFormat)
{
	int iNumInt;
	int iNumDec;
	std::string sDecimalSign;
	std::string sTemp;
	int iTemp;

	// reset error code
	iBPError = 0;

	// initialize values
	iNumInt = 0;
	iNumDec = 0;
	sDecimalSign = "";

	// copy format
	sTemp = sBP_SetFormat;

	// search for number of char for the integer part
	iTemp = sTemp.find_first_not_of ("9");
	if (iTemp >= 0)
		iNumInt = iTemp;
	else
		iNumInt = sTemp.length ();

	// assign decimal sign
	if (iTemp >= 0)
		sDecimalSign = sTemp [iTemp];

	// search for number of char for the decimal part
	sTemp = sTemp.substr (iTemp + 1);
	iTemp = sTemp.find_first_not_of ("8");
	if (iTemp >= 0)
		iNumDec = iTemp;
	else
		iNumDec = sTemp.length ();

	// store format
	BP_Currency::setformat (iNumInt, iNumDec, sDecimalSign,  "",  "",  "", "", false);

}



//-----------------------------------------------------------------
//!    BP_Currency::setstdformat : set standard format            !
//-----------------------------------------------------------------
//! iBP_SetNumInt : character to display before decimal sign      !
//! iBP_SetNumDec : character to display after decimal sign       !
//! sBP_SetFormat : name of the standard format                   !
//!                                                               !
//! return : none                                                 !
//-----------------------------------------------------------------
//!  standard ! decimal ! kilo ! currency ! leading ! null ! trail!
//!   format  !  sign   ! sign !  symbol  !  char   ! fmt  ! zeros!
//! --------------------------------------------------------------!
//!     US         .      none     none      none     none    no  !
//!     US$        .       ,        $        none     none    no  !
//!     FR         ,      none     none      none     none    no  !
//!     FR$        ,       .       euro      none     none    no  !
//-----------------------------------------------------------------
//!     BP_Date::geterror() return error code                     !
//-----------------------------------------------------------------
BP_Currency::setstdformat (const int iBP_SetNumInt,
						   const int iBP_SetNumDec,
						   const std::string sBP_SetFormat)
{
	// reset error code
	iBPError = 0;
	
	if (sBP_SetFormat == "US")
		BP_Currency::setformat (iBP_SetNumInt, iBP_SetNumDec, ".",  "",  "",  "", "", false);
	else if (sBP_SetFormat == "US$")
		BP_Currency::setformat (iBP_SetNumInt, iBP_SetNumDec, ".",  ",",  "$",  "", "", false);
	else if (sBP_SetFormat == "FR")
		BP_Currency::setformat (iBP_SetNumInt, iBP_SetNumDec, ",",  "",  "",  "", "", false);
	else if (sBP_SetFormat == "FR$")
		BP_Currency::setformat (iBP_SetNumInt, iBP_SetNumDec, ",",  ".",  "euro",  "", "", false);
	else
		iBPError = 2;

}



//-----------------------------------------------------------------
//!    BP_Currency::getformat : get format of currency            !
//-----------------------------------------------------------------
//! return : BP_STRUCT_FORMAT structure                           !
//-----------------------------------------------------------------
BP_STRUCT_FORMAT BP_Currency::getformat (void)
{

	BP_STRUCT_FORMAT BPFormat;

	BPFormat.iBP_NumInt = BP_Fmt.iBP_NumInt;
	BPFormat.iBP_NumDec = BP_Fmt.iBP_NumDec;
	BPFormat.sBP_DecimalSign = BP_Fmt.sBP_DecimalSign;
	BPFormat.sBP_KiloSign = BP_Fmt.sBP_KiloSign;
	BPFormat.sBP_CurrencySymbol = BP_Fmt.sBP_CurrencySymbol;
	BPFormat.sBP_LeadChar = BP_Fmt.sBP_LeadChar;
	BPFormat.sBP_Null = BP_Fmt.sBP_Null;
	BPFormat.bBP_TrailZero = BP_Fmt.bBP_TrailZero;

	return BPFormat;

}



//-----------------------------------------------------------------
//!    BP_Currency::+ : overload operator                         !
//-----------------------------------------------------------------
BP_Currency BP_Currency::operator + (const BP_Currency& BPright)
{
	BP_Currency curResult;		// result currency
	BP_Currency curLeft;
	BP_Currency curRight;
	BP_Currency curSave;


	// reset error code
	iBPError = 0;

	// one value is null -> result is null
	if (BP_Cur.bBP_Null == true || BPright.BP_Cur.bBP_Null == true)
	{
		curResult.setnull ();
		return curResult;
	}

	// save format
	curSave = *this;

	// add integer and decimal part
	if (this->BP_Cur.bBP_Negative == BPright.BP_Cur.bBP_Negative)
	{
		// both currency have the same sign
		curResult.BP_Cur.lBP_Integer = this->BP_Cur.lBP_Integer + BPright.BP_Cur.lBP_Integer;
		curResult.BP_Cur.lBP_Decimal = this->BP_Cur.lBP_Decimal + BPright.BP_Cur.lBP_Decimal;
		if (curResult.BP_Cur.lBP_Decimal > BP_DECIMALX2)
			curResult.BP_Cur.lBP_Decimal -= BP_DECIMAL;

		// overload
		if (curResult.BP_Cur.lBP_Decimal >= BP_DECIMALX2)
		{
			curResult.BP_Cur.lBP_Integer++;
			curResult.BP_Cur.lBP_Decimal -= BP_DECIMAL;

			if (curResult.BP_Cur.lBP_Decimal == BP_DECIMAL)
				curResult.BP_Cur.lBP_Decimal = 0;
		}

		// define sign
		if (this->BP_Cur.bBP_Negative == true)
			curResult.BP_Cur.bBP_Negative = true;
	}
	else if (this->BP_Cur.bBP_Negative == false && BPright.BP_Cur.bBP_Negative == true)
	{
		// left currency is positive and right is negative
		curRight = BPright;
		curRight.BP_Cur.bBP_Negative = false;
		curResult = *this - curRight;
	}
	else if (this->BP_Cur.bBP_Negative == true && BPright.BP_Cur.bBP_Negative == false)
	{
		// left currency is negative and right is positive
		curLeft = *this;
		curLeft.BP_Cur.bBP_Negative = false;
		curRight = BPright;
		curResult = curRight - curLeft;
	}


	// currency is not null
	curResult.BP_Cur.bBP_Null = false;


	// overload of integer part
	if (curResult.BP_Cur.lBP_Integer >= BP_MAX_INT)
	{
		curResult.setnull ();
		curResult.iBPError = 7;

		return curResult;
	}

	// overload of decimal part - internal error
	if (curResult.BP_Cur.lBP_Decimal > BP_MAX_DEC)
	{
		curResult.setnull ();
		curResult.iBPError = 8;

		return curResult;
	}

	// restore format
	curResult.BP_Fmt = curSave.BP_Fmt;

	return curResult;

}



//-----------------------------------------------------------------
//!    BP_Currency::- : overload operator                         !
//-----------------------------------------------------------------
BP_Currency BP_Currency::operator - (const BP_Currency& BPright)
{
	BP_Currency curResult;		// result currency
	BP_Currency curLeft;
	BP_Currency curRight;
	BP_Currency curSave;

	// reset error code
	iBPError = 0;

	// one value is null -> result is null
	if (BP_Cur.bBP_Null == true || BPright.BP_Cur.bBP_Null == true)
	{
		curResult.setnull ();
		return curResult;
	}


	// save format
	curSave = *this;

	if (this->BP_Cur.bBP_Negative == true && BPright.BP_Cur.bBP_Negative == true)
	{
		// both currency are negative
		curRight = BPright;
		curRight.BP_Cur.bBP_Negative = false;
		curResult = *this + curRight;
	}
	else if (this->BP_Cur.bBP_Negative == true && BPright.BP_Cur.bBP_Negative == false)
	{
		// left currency is negative and right is positive
		curLeft = *this;
		curLeft.BP_Cur.bBP_Negative = false;
		curRight = BPright;
		curRight.BP_Cur.bBP_Negative = false;

		curResult = curLeft + curRight;
		curResult.BP_Cur.bBP_Negative = true;
	}
	else if (this->BP_Cur.bBP_Negative == false && BPright.BP_Cur.bBP_Negative == true)
	{
		// left currency is positive and right is negative
		curRight = BPright;
		curRight.BP_Cur.bBP_Negative = false;

		curResult = *this + curRight;
	}
	else
	{
		// both currency are positive
		curResult.BP_Cur.lBP_Integer = this->BP_Cur.lBP_Integer - BPright.BP_Cur.lBP_Integer;

		if (curResult.BP_Cur.lBP_Integer < 0 ||
			(curResult.BP_Cur.lBP_Integer == 0 && this->BP_Cur.lBP_Decimal < BPright.BP_Cur.lBP_Decimal))
		{
			curResult.BP_Cur.lBP_Integer *= -1;
			curResult.BP_Cur.bBP_Negative = true;
		}


		// compute decimal part
		if (this->BP_Cur.lBP_Decimal < BPright.BP_Cur.lBP_Decimal)
		{
			if (this->BP_Cur.lBP_Integer > BPright.BP_Cur.lBP_Integer)
			{
				curResult.BP_Cur.lBP_Integer--;
				if (this->BP_Cur.lBP_Decimal == 0)
				{
					curResult.BP_Cur.lBP_Decimal = BP_DECIMALX2;
					curResult.BP_Cur.lBP_Decimal += BP_DECIMAL;
					curResult.BP_Cur.lBP_Decimal -= BPright.BP_Cur.lBP_Decimal;
				}
				else
				{
					curResult.BP_Cur.lBP_Decimal = BP_DECIMALX2 + this->BP_Cur.lBP_Decimal;
					curResult.BP_Cur.lBP_Decimal -= BPright.BP_Cur.lBP_Decimal;
				}
			}
			else
				if (this->BP_Cur.lBP_Decimal != 0)
				{
					curResult.BP_Cur.lBP_Decimal = (BP_DECIMAL + BPright.BP_Cur.lBP_Decimal)
						- this->BP_Cur.lBP_Decimal;
				}
				else
				{
					curResult.BP_Cur.lBP_Decimal = BPright.BP_Cur.lBP_Decimal
						- this->BP_Cur.lBP_Decimal;
				}
		}
		else if (this->BP_Cur.lBP_Decimal > BPright.BP_Cur.lBP_Decimal)
		{
			if (this->BP_Cur.lBP_Integer < BPright.BP_Cur.lBP_Integer)
			{
				curResult.BP_Cur.lBP_Integer--;
				if (this->BP_Cur.lBP_Decimal == 0)
				{
					curResult.BP_Cur.lBP_Decimal = BP_DECIMALX2;
					curResult.BP_Cur.lBP_Decimal += BP_DECIMAL;
				}
				else
				{
					if (BPright.BP_Cur.lBP_Decimal != 0)
					{
						curResult.BP_Cur.lBP_Decimal = BP_DECIMALX2 + BPright.BP_Cur.lBP_Decimal;
						curResult.BP_Cur.lBP_Decimal -=	this->BP_Cur.lBP_Decimal;
					}
					else
					{
						curResult.BP_Cur.lBP_Decimal = BP_DECIMALX2;
						curResult.BP_Cur.lBP_Decimal += BP_DECIMAL;
						curResult.BP_Cur.lBP_Decimal -= this->BP_Cur.lBP_Decimal;
					}
				}
			}
			else
			{
				if (BPright.BP_Cur.lBP_Decimal != 0)
				{
					curResult.BP_Cur.lBP_Decimal = (BP_DECIMAL + this->BP_Cur.lBP_Decimal)
								- BPright.BP_Cur.lBP_Decimal;
				}
				else
				{
					curResult.BP_Cur.lBP_Decimal = this->BP_Cur.lBP_Decimal
								- BPright.BP_Cur.lBP_Decimal;
				}
			}
		}
	}



	// currency is not null
	curResult.BP_Cur.bBP_Null = false;


	// overload of integer part
	if (curResult.BP_Cur.lBP_Integer >= BP_MAX_INT)
	{
		iBPError = 7;

		curResult.setnull ();
		return curResult;
	}

	// overload of decimal part - internal error
	if (curResult.BP_Cur.lBP_Decimal >= BP_MAX_DEC)
	{
		iBPError = 8;

		curResult.setnull ();
		return curResult;
	}

	// restore format
	curResult.BP_Fmt = curSave.BP_Fmt;

	return curResult;

}



//-----------------------------------------------------------------
//!    BP_Currency::+= : overload operator                        !
//-----------------------------------------------------------------
BP_Currency BP_Currency::operator += (const BP_Currency& BPright)
{

	// error code is reset in addition

	*this = *this + BPright;

	return *this;

}



//-----------------------------------------------------------------
//!    BP_Currency::-= : overload operator                        !
//-----------------------------------------------------------------
BP_Currency BP_Currency::operator -= (const BP_Currency& BPright)
{

	// error code is reset in addition

	*this = *this - BPright;

	return *this;

}



//-----------------------------------------------------------------
//!    BP_Currency::> : overload operator                         !
//-----------------------------------------------------------------
bool BP_Currency::operator > (const BP_Currency& BPright)
{
	// reset error code
	iBPError = 0;

	// one currency is null -> result false
	if (BP_Cur.bBP_Null == true || BPright.BP_Cur.bBP_Null == true)
		return false;

	// this is negative and BPright is positive
	if (BP_Cur.bBP_Negative == true && BPright.BP_Cur.bBP_Negative == false)
		return false;

	// this is positive and BPright is negative
	if (BP_Cur.bBP_Negative == false && BPright.BP_Cur.bBP_Negative == true)
		return true;

	// integer part of this is greater than BPright
	if (((BP_Cur.lBP_Integer > BPright.BP_Cur.lBP_Integer) &&
		(BP_Cur.bBP_Negative == false)) ||
		((BP_Cur.lBP_Integer < BPright.BP_Cur.lBP_Integer) &&
		(BP_Cur.bBP_Negative == true)))
			return true;
	// integer part are equal, look decimal part
	else if (BP_Cur.lBP_Integer == BPright.BP_Cur.lBP_Integer)
	{
		if ((BP_Cur.lBP_Decimal > BPright.BP_Cur.lBP_Decimal) &&
			(BP_Cur.bBP_Negative == false))
				return true;
		else if ((BP_Cur.lBP_Decimal < BPright.BP_Cur.lBP_Decimal) &&
			(BP_Cur.bBP_Negative == true))
				return true;
	}

	return false;
}



//-----------------------------------------------------------------
//!    BP_Currency::>= : overload operator                        !
//-----------------------------------------------------------------
bool BP_Currency::operator >= (const BP_Currency& BPright)
{
	// reset error code
	iBPError = 0;

	// currency are equal
	if (*this == BPright)
		return true;

	// use > operator
	if (*this > BPright)
		return true;

	return false;
}



//-----------------------------------------------------------------
//!    BP_Currency::< : overload operator                         !
//-----------------------------------------------------------------
bool BP_Currency::operator < (const BP_Currency& BPright)
{
	// reset error code
	iBPError = 0;

	// currency are equal
	if (*this == BPright)
		return false;

	// use > operator
	if (*this > BPright)
		return false;

	return true;
}



//-----------------------------------------------------------------
//!    BP_Currency::<= : overload operator                        !
//-----------------------------------------------------------------
bool BP_Currency::operator <= (const BP_Currency& BPright)
{
	// reset error code
	iBPError = 0;

	// use > operator
	if (*this > BPright)
		return false;

	return true;
}



//-----------------------------------------------------------------
//!    BP_Currency::== : overload operator                        !
//-----------------------------------------------------------------
bool BP_Currency::operator == (const BP_Currency& BPright)
{
	// reset error code
	iBPError = 0;

	// currency are equal
	if ((this->BP_Cur.lBP_Integer == BPright.BP_Cur.lBP_Integer) &&
		(this->BP_Cur.lBP_Decimal == BPright.BP_Cur.lBP_Decimal) &&
		(this->BP_Cur.bBP_Negative == BPright.BP_Cur.bBP_Negative) &&
		(this->BP_Cur.bBP_Null == BPright.BP_Cur.bBP_Null))
			return true;

	return false;
}



//-----------------------------------------------------------------
//!    BP_Currency::!= : overload operator                        !
//-----------------------------------------------------------------
bool BP_Currency::operator != (const BP_Currency& BPright)
{
	// error code is reset in ==

	// currency are equal
	if (*this == BPright)
		return false;

	return true;
}



//-----------------------------------------------------------------
//!    BP_Currency::++ : overload operator                        !
//!        add 1 to integer part, don't change decimal part       !
//-----------------------------------------------------------------
BP_Currency BP_Currency::operator ++ (int BPright)
{
	BP_Currency curResult;		// result currency

	// reset error code
	iBPError = 0;


	curResult = *this;
	if (curResult.BP_Cur.lBP_Integer >= (BP_MAX_INT-1))
	{
		curResult.setnull ();
		return curResult;
	}


	if (curResult.BP_Cur.bBP_Negative == false)
		curResult.BP_Cur.lBP_Integer++;
	else
	{
		if (curResult.BP_Cur.lBP_Integer == 0)
		{
			curResult = *this + 1;
		}
		else
			curResult.BP_Cur.lBP_Integer--;

	}


	*this = curResult;
	return *this;
}



//-----------------------------------------------------------------
//!    BP_Currency::-- : overload operator                        !
//!      substract 1 from integer part, don't change decimal part !
//-----------------------------------------------------------------
BP_Currency BP_Currency::operator -- (int BPright)
{
	BP_Currency curResult;		// result currency

	// reset error code
	iBPError = 0;


	curResult = *this;
	if (curResult.BP_Cur.lBP_Integer <= (-BP_MAX_INT+1))
	{
		curResult.setnull ();
		return curResult;
	}


	if (curResult.BP_Cur.bBP_Negative == true)
		curResult.BP_Cur.lBP_Integer++;
	else
	{
		curResult.BP_Cur.lBP_Integer--;

		if (curResult.BP_Cur.lBP_Integer == -1)
		{
			curResult.BP_Cur.bBP_Negative = true;
			curResult.BP_Cur.lBP_Integer = 1;
		}
	}


	*this = curResult;
	return *this;
}



//-----------------------------------------------------------------
//!    BP_Currency::set : convert long into currency              !
//-----------------------------------------------------------------
//! lBP_Long : long value                                         !
//!																  !
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Currency::set (const long lBP_Long)
{
	// reset error code
	iBPError = 0;

	if (lBP_Long >= 0)
	{
		BP_Cur.lBP_Integer = lBP_Long;
		BP_Cur.bBP_Negative = false;
	}
	else
	{
		BP_Cur.lBP_Integer = -lBP_Long;
		BP_Cur.bBP_Negative = true;
	}

	// reset decimal part
	BP_Cur.lBP_Decimal = 0;

	// currency is not null
	BP_Cur.bBP_Null = false;

}


//-----------------------------------------------------------------
//!    BP_Currency::+ : overload operator                         !
//-----------------------------------------------------------------
BP_Currency operator + (const BP_Currency& BPleft, const long& BPright)
{
	BP_Currency curResult;		// result currency
	BP_Currency curLeft;
	BP_Currency curRight;
	BP_Currency curSave;


	// save format
	curSave = BPleft;

	// currency null -> result is null
	if (BPleft.BP_Cur.bBP_Null == true)
	{
		curResult.setnull ();
		return curResult;
	}

	curLeft = BPleft;
	curRight.set (BPright);
	curResult = curLeft + curRight;

	// restore format
	curResult.BP_Fmt = curSave.BP_Fmt;

	return curResult;

}



//-----------------------------------------------------------------
//!    BP_Currency::+ : overload operator                         !
//-----------------------------------------------------------------
BP_Currency operator + (const long& BPleft, const BP_Currency& BPright)
{
	BP_Currency curResult;		// result currency
	BP_Currency curLeft;
	BP_Currency curRight;
	BP_Currency curSave;


	// save format
	curSave = BPright;

	// currency null -> result is null
	if (BPright.BP_Cur.bBP_Null == true)
	{
		curResult.setnull ();
		return curResult;
	}

	curLeft.set (BPleft);
	curRight = BPright;
	curResult = curLeft + curRight;

	// restore format
	curResult.BP_Fmt = curSave.BP_Fmt;

	return curResult;

}



//-----------------------------------------------------------------
//!    BP_Currency::- : overload operator                         !
//-----------------------------------------------------------------
BP_Currency operator - (const BP_Currency& BPleft, const long& BPright)
{
	BP_Currency curResult;		// result currency
	BP_Currency curLeft;
	BP_Currency curRight;
	BP_Currency curSave;


	// save format
	curSave = BPleft;

	// currency null -> result is null
	if (BPleft.BP_Cur.bBP_Null == true)
	{
		curResult.setnull ();
		return curResult;
	}

	curLeft = BPleft;
	curRight.set (BPright);
	curResult = curLeft - curRight;

	// restore format
	curResult.BP_Fmt = curSave.BP_Fmt;

	return curResult;

}



//-----------------------------------------------------------------
//!    BP_Currency::- : overload operator                         !
//-----------------------------------------------------------------
BP_Currency operator - (const long& BPleft, const BP_Currency& BPright)
{
	BP_Currency curResult;		// result currency
	BP_Currency curLeft;
	BP_Currency curRight;
	BP_Currency curSave;


	// save format
	curSave = BPright;

	// currency null -> result is null
	if (BPright.BP_Cur.bBP_Null == true)
	{
		curResult.setnull ();
		return curResult;
	}

	curLeft.set (BPleft);
	curRight = BPright;
	curResult = curLeft - curRight;

	// restore format
	curResult.BP_Fmt = curSave.BP_Fmt;

	return curResult;

}



//-----------------------------------------------------------------
//!    BP_Currency::== : overload operator                        !
//-----------------------------------------------------------------
bool BP_Currency::operator == (const long& BPright)
{
	long lLong;
	bool bNegative;

	// reset error code
	iBPError = 0;

	if (BPright >= 0)
	{
		lLong = BPright;
		bNegative = false;
	}
	else
	{
		lLong = -BPright;
		bNegative = true;
	}

	// currency are equal
	if ((this->BP_Cur.lBP_Integer == BPright) &&
		(this->BP_Cur.lBP_Decimal == 0) &&
		(this->BP_Cur.bBP_Negative == bNegative) &&
		(this->BP_Cur.bBP_Null != true))
			return true;

	return false;
}



//-----------------------------------------------------------------
//!    BP_Currency::!= : overload operator                        !
//-----------------------------------------------------------------
bool BP_Currency::operator != (const long& BPright)
{
	// error code is reset in ==

	// currency are equal
	if (*this == BPright)
		return false;

	return true;
}



//-----------------------------------------------------------------
//!    BP_Currency::> : overload operator                         !
//-----------------------------------------------------------------
bool BP_Currency::operator > (const long& BPright)
{
	long lLong;


	// reset error code
	iBPError = 0;

	// currency is null -> result false
	if (BP_Cur.bBP_Null == true)
		return false;

	// this is negative and BPright is positive
	if (BP_Cur.bBP_Negative == true && BPright >= 0)
		return false;

	// this is positive and BPright is negative
	if (BP_Cur.bBP_Negative == false && BPright < 0)
		return true;


	if (BPright >= 0)
		lLong = BPright;
	else
		lLong = -BPright;

	// integer part of this is greater than BPright
	if (((BP_Cur.lBP_Integer > lLong) && (BP_Cur.bBP_Negative == false)) ||
		((BP_Cur.lBP_Integer < lLong) && (BP_Cur.bBP_Negative == true)))
			return true;
	// integer part are equal, look decimal part
	else if (BP_Cur.lBP_Integer == lLong)
	{
		if ((BP_Cur.lBP_Decimal > 0) && (BP_Cur.bBP_Negative == false))
				return true;
	}

	return false;
}



//-----------------------------------------------------------------
//!    BP_Currency::>= : overload operator                        !
//-----------------------------------------------------------------
bool BP_Currency::operator >= (const long& BPright)
{
	// reset error code
	iBPError = 0;

	// currency are equal
	if (*this == BPright)
		return true;

	// use > operator
	if (*this > BPright)
		return true;

	return false;
}



//-----------------------------------------------------------------
//!    BP_Currency::< : overload operator                         !
//-----------------------------------------------------------------
bool BP_Currency::operator < (const long& BPright)
{
	// reset error code
	iBPError = 0;

	// currency are equal
	if (*this == BPright)
		return false;

	// use > operator
	if (*this > BPright)
		return false;

	return true;
}



//-----------------------------------------------------------------
//!    BP_Currency::<= : overload operator                        !
//-----------------------------------------------------------------
bool BP_Currency::operator <= (const long& BPright)
{
	// reset error code
	iBPError = 0;

	// use > operator
	if (*this > BPright)
		return false;

	return true;
}



//-----------------------------------------------------------------
//!    BP_Currency::<< : overload ostream operator                !
//-----------------------------------------------------------------
std::ostream & operator << (std::ostream & os, const BP_Currency& BPright)
{
	BP_Currency curSave;

	curSave = BPright;
    os << curSave.str ();
    return os;
}

